@@ -12,6 +12,7 @@
 #' @param tzone a character string containing the time zone to convert to. R
 #'   must recognize the name contained in the string as a time zone on your
 #'   system.
+#' @param ... Parameters passed to other methods.
 #' @return a POSIXct object in the updated time zone
 #' @keywords chron manip
 #' @seealso [force_tz()]
@@ -19,9 +20,16 @@
 #' x <- ymd_hms("2009-08-07 00:00:01", tz = "America/New_York")
 #' with_tz(x, "GMT")
 #' @export
-with_tz <- function(time, tzone = "") {
-  if (!C_valid_tz(tzone))
+with_tz <- function(time, tzone = "", ...) {
+    UseMethod("with_tz")
+}
+
+#' @rdname with_tz
+#' @export
+with_tz.default <- function(time, tzone = "", ...) {
+  if (!timechange:::C_valid_tz(tzone)) {
     warning(sprintf("Unrecognized time zone '%s'", tzone))
+  }
   if (is.data.frame(time)) {
     for (nm in names(time)) {
       if (is.POSIXt(time[[nm]])) {
@@ -63,23 +71,43 @@ with_tz <- function(time, tzone = "") {
 #' @param tzone a character string containing the time zone to convert to. R
 #'   must recognize the name contained in the string as a time zone on your
 #'   system.
-#' @param roll logical. If TRUE, and `time` falls into the DST-break, assume
-#'   the next valid civil time, otherwise return NA. See examples.
+#' @param roll deprecated, same as `roll_dst` parameter.
+#' @param ... Parameters passed to other methods.
 #' @return a POSIXct object in the updated time zone
 #' @keywords chron manip
 #' @seealso [with_tz()], [local_time()]
+#' @inheritParams timechange::time_add
 #' @examples
 #' x <- ymd_hms("2009-08-07 00:00:01", tz = "America/New_York")
 #' force_tz(x, "UTC")
 #' force_tz(x, "Europe/Amsterdam")
 #'
 #' ## DST skip:
-#'
 #' y <- ymd_hms("2010-03-14 02:05:05 UTC")
-#' force_tz(y, "America/New_York", roll=FALSE)
-#' force_tz(y, "America/New_York", roll=TRUE)
+#' force_tz(y, "America/New_York", roll_dst = "NA")
+#' force_tz(y, "America/New_York", roll_dst = "pre")
+#' force_tz(y, "America/New_York", roll_dst = "boundary")
+#' force_tz(y, "America/New_York", roll_dst = "post")
+#'
+#' ## DST repeat
+#' y <- ymd_hms("2014-11-02 01:35:00", tz = "UTC")
+#' force_tz(y, "America/New_York", roll_dst = "NA")
+#' force_tz(y, "America/New_York", roll_dst = "pre")
+#' force_tz(y, "America/New_York", roll_dst = "boundary")
+#' force_tz(y, "America/New_York", roll_dst = "post")
+#'
+#' ## DST skipped and repeated
+#' y <- ymd_hms("2010-03-14 02:05:05 UTC", "2014-11-02 01:35:00", tz = "UTC")
+#' force_tz(y, "America/New_York", roll_dst = c("NA", "pre"))
+#' force_tz(y, "America/New_York", roll_dst = c("boundary", "post"))
 #' @export
-force_tz <- function(time, tzone = "", roll = FALSE) {
+force_tz <- function(time, tzone = "", ...) {
+    UseMethod("force_tz")
+}
+
+#' @rdname force_tz
+#' @export
+force_tz.default <- function(time, tzone = "", roll_dst = c("NA", "post"), roll = NULL, ...) {
   tzone <- as.character(tzone)
   if (is.data.frame(time)) {
     for (nm in names(time)) {
@@ -89,12 +117,13 @@ force_tz <- function(time, tzone = "", roll = FALSE) {
     }
     time
   } else {
-    if (is.POSIXct(time))
-      cpp_force_tz(time, tz = tzone, roll)
-    else if (is.Date(time))
-      cpp_force_tz(date_to_posix(time), tz = tzone, roll)
-    else {
-      out <- cpp_force_tz(as.POSIXct(time, tz = tz(time)), tz = tzone, roll)
+    roll_dst <- normalize_roll_dst(roll_dst, roll)
+    if (is.POSIXct(time)) {
+      timechange::time_force_tz(time, tz = tzone, roll_dst = roll_dst)
+    } else if (is.Date(time)) {
+      timechange::time_force_tz(date_to_posix(time), tz = tzone, roll_dst = roll_dst)
+    } else {
+      out <- timechange::time_force_tz(as.POSIXct(time, tz = tz(time)), tz = tzone, roll_dst = roll_dst)
       reclass_date(out, time)
     }
   }
@@ -116,15 +145,16 @@ force_tz <- function(time, tzone = "", roll = FALSE) {
 #' x <- ymd_hms("2009-08-07 00:00:01")
 #' force_tzs(x, tzones = c("America/New_York", "Europe/Amsterdam"))
 #' @export
-force_tzs <- function(time, tzones, tzone_out = "UTC", roll = FALSE) {
-  if (length(tzones) < length(time))
+force_tzs <- function(time, tzones, tzone_out = "UTC", roll_dst = c("NA", "post"), roll = NULL) {
+  if (length(tzones) < length(time)) {
     tzones <- rep_len(tzones, length(time))
-  else if (length(tzones) > length(time)) {
+  } else if (length(tzones) > length(time)) {
     attr <- attributes(time)
     time <- rep_len(time, length(tzones))
     attributes(time) <- attr
   }
-  out <- cpp_force_tzs(as.POSIXct(time), tzones, tzone_out, roll)
+  roll_dst <- normalize_roll_dst(roll_dst, roll)
+  out <- timechange::time_force_tz(as.POSIXct(time), tzones, tzone_out, roll_dst)
   reclass_date(out, time)
 }
 
@@ -149,17 +179,5 @@ force_tzs <- function(time, tzones, tzone_out = "UTC", roll = FALSE) {
 #' local_time(x, c("America/New_York", "Europe/Amsterdam", "Asia/Shanghai"), unit = "hours")
 #' @export
 local_time <- function(dt, tz = NULL, units = "secs") {
-  if (is.null(tz))
-    tz <- tz(dt)
-  if (length(tz) < length(dt))
-    tz <- rep_len(tz, length(dt))
-  else if (length(tz) > length(dt)) {
-    attr <- attributes(dt)
-    dt <- rep_len(dt, length(tz))
-    attributes(dt) <- attr
-  }
-  secs <- cpp_local_time(as.POSIXct(dt), tz)
-  out <- structure(secs, units = "secs", class = "difftime")
-  units(out) <- units
-  out
+  timechange::time_clock_at_tz(dt, tz, units)
 }
